#-
@*                                               *****************************************************************
@*                                               * FILE:        calculateMatrix.frm                              *
@*                                               * written by:  Alexander Hentschel                              *
@*                                               *              Humboldt University Berlin                       *
@*                                               *              Alexander.Hentschel@physik.hu-berlin.de          *
@*                                               *              July & August 2006                               *
@*                                               *****************************************************************

@ COMMENT:
@ please set "commentchar @" in file "form.set" indicating that comments begin with "@" 
@ but in "form.set" the character "*" remains as the comment sign

@ DESCRIPTION:
@ if terms in the Hamiltonian consist of many creation/anhilation operators output may get very large
@ in this case a seperation into a product of sums, where each sum contains terms of one subsate 
@ might be helpful
@ to enable this feature use: #define SperateMatrixElementBySubstates
@ #define SeperateMatrixElementBySubstates 

#include define_states.load
#include Hamiltonian.h
#include calculateMatrixElement.prc
#include export.prc

@ #define WriteMatrixElementToScreen

FunPowers allfunpowers;
format 150;
  
#procedure OutputControl
@  Bracket FGminusGF,FFminusGG;
@  .sort 
@  collect TRY;
@  Keep Brackets;
@  id many c = -b -a;
@  id TRY(NummerX?) = NummerX;
@  Bracket InvSqrtOmega, Kappa4;
#endprocedure


#procedure Simplifications
  id many InvSqrtOmega(NummerX?) = Omega(NummerX)^(-1/2); 
  id many SqrtOmega(NummerX?) =  Omega(NummerX)^(1/2);
  repeat;
     id many DeltaSum(Index0?,Index1?,?Index2) = DeltaSum(Index0+Index1,?Index2);
  endrepeat;
  id many DeltaSum(0) = 1;
  id many DeltaSum(Index0?) = DiscreteDelta(Index0);
#endprocedure

#procedure calculateMatrixAndExportToMathematica(HamiltonianFunction,HamiltonianOperator,MatrixName,ElementName)
   #call BeginMatrixExportToMathematica('MatrixName','ExportMatrixFileName','ExportStatesEquationFileName')
   .sort
   #include matrix_calculation.load
   .sort
   #call EndMatrixExportToMathematica('ExportMatrixFileName','ExportStatesEquationFileName')
#endprocedure

@~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ Main Programm ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include effective_Hamiltonian.def

#ifdef 'MathOutputFile'
   #write  " CALCULATING HAMILTONIAN MATRIX (done by FORM-program calculateMatrix.frm):"
   format mathematica;

@ Initialisierung
   #create <'ExportMatrixFileName'>
   #create <'ExportStatesEquationFileName'>

@ Berechnung
   #define DummyElementName "E"
   #define DummyHamiltonianPartName "Part"
   #do PartX = 1, 'HamiltonianParts'
     #ifdef `VERBOSE'
         #write  "   MESSAGE: calculating Part'PartX' of the hamiltonian "
     #endif
     #call calculateMatrixAndExportToMathematica(HamiltonianFunctionPart'PartX',HamiltonianOperatorPart'PartX','DummyHamiltonianPartName''PartX','DummyElementName''PartX'x)
   #enddo
   
@ Abschluss
   #call ExportToMathematicaEigenvalueCalculation('ExportMatrixFileName','HamiltonianParts','DummyHamiltonianPartName')   
   #call completeExportToMathematica('ExportMathematicaClosure')
   #close <'ExportMatrixFileName'>
   #close <'ExportStatesEquationFileName'>
   
   #write "   MESSAGE: calculation done"  
   #write "            result written to file \"'MathOutputFile'\""  
   #write ""
   
#endif
.end



















@ End of FORM-FILE