@*                                               *****************************************************************
@*                                               * FILE:        calculateMatrixElement.prc                       *
@*                                               * written by:  Alexander Hentschel                              *
@*                                               *              Humboldt University Berlin                       *
@*                                               *              Alexander.Hentschel@physik.hu-berlin.de          *
@*                                               *              July 2006                                        *
@*                                               ****************************************************************{


@*****************************************************************************************************************
@***********************************   Module calculateMatrixElement   *******************************************
@*****************************************************************************************************************
@* provides the following FORM procedures:			 						 *
@*    culateFermSubstate(STATE)											 *
@*    calculateBosSubstate(STATE)										 *
@*    OrderMatrixElement					 						 *
@*    CalculateElement												 *
@*    BeginNewHamiltonianLine											 *
@*    renameBra													 *
@* ------------------------------------------------------------------------------------------------------------- *
@* provides the following symbol and function Definitions:	 						 *
@*    see global definitions			 								 *
@*****************************************************************************************************************

@*********************************************  global definitions  **********************************************
@   #define WriteMatrixElementToScreen
   
   NFunction Op,Op1,Op2,St,St1,St2;
   CFunction delta, func;
   AutoDeclare Symbol ExNumber;
   AutoDeclare Symbol Index;
   Symbol NummerX,NummerY,NummerY;
   Symbol GNumber, GNumber1,GNumber2;
@*****************************************  end of global Definitions ********************************************

@*****************************  Functions and Symbols for use in Procedures ONLY ! *******************************
@*****   				Please don't use outside this module       			     *****
   CFunction SymbolContainer
@***********************************  end of procedure Functions and Symbols ************************************}


@************************** Procedure calculateFermSubstate /calculateBosSubstate *******************************{
@* USE: 	calculateFermSubstate(STATE), calculateBosSubstate(STATE)					 *
@* DESCRIPTION:							 						 *
@* 	calculates the action of fermionic / bosinic operators on State 'STATE' 				 *
@* 	since the Hamiltonian should occur normalordered the action of anhilation operators is calculated first	 *
@* 	after all anhilation and creation operators haven been worked out the suitable substate of the bra 	 *
@* 	vector is  used to skip al Terms not contributing to the matrix element 				 *
@*****************************************************************************************************************
#procedure calculateFermSubstate(STATE)
 Bracket S'STATE'; .sort
 Keep Brackets;
 repeat;
   id Op?FermAnhilators(Index?) * S'STATE'(GNumber?,'ExBracket') = delta(Index,ModeIndices(1))*Sqrt(ExNumber1)* S'STATE'(GNumber-1,'decIndex1')
      #do i=2,'NumberOfModes'
        +  delta(Index,ModeIndices('i'))*sign_(ExNumber1+...+ExNumber{'i'-1})*Sqrt(ExNumber'i')* S'STATE'(GNumber-1,'decIndex{'i'}')
      #enddo
      ;
   id Sqrt(0)=0;   id Sqrt(1)=1; id Sqrt(NummerX?)^2 = NummerX;
 endrepeat;
 repeat;
   id Op?FermCreators(Index?) * S'STATE'(GNumber?,'ExBracket') = delta(Index,ModeIndices(1))*Sqrt(ExNumber1+1)* S'STATE'(GNumber+1,'incIndex1')
      #do i=2,'NumberOfModes'
         + delta(Index,ModeIndices('i'))*sign_(ExNumber1+...+ExNumber{'i'-1})*Sqrt(ExNumber'i'+1)* S'STATE'(GNumber+1,'incIndex{'i'}')
      #enddo
      ;
   id Sqrt(0)=0;   id Sqrt(1)=1; id Sqrt(NummerX?)^2 = NummerX;
 endrepeat;
 .sort
 id Bra'STATE'(GNumber?,'ExBracket') * S'STATE'(GNumber?,'ExBracket') = 1;
   also S'STATE'(GNumber?,'ExBracket') = 0;
 #ifdef 'SeperateMatrixElementBySubstates'
   Bracket 'OperatorBrackets';
   .sort
   collect 'STATE'Terms;
 #endif
#endprocedure

#procedure calculateBosSubstate(STATE)
 Bracket S'STATE'; .sort
 Keep Brackets;
 repeat;
   id Op?BosAnhilators(Index?) * S'STATE'(GNumber?,'ExBracket') = 
      #do i=1,'NumberOfModes'
         + delta(Index,ModeIndices('i'))*Sqrt(ExNumber'i')* S'STATE'(GNumber,'decIndex{'i'}')
      #enddo
      ;
   id Sqrt(0)=0;   id Sqrt(1)=1; id Sqrt(NummerX?)^2 = NummerX;
 endrepeat;
 repeat;
   id Op?BosCreators(Index?) * S'STATE'(GNumber?,'ExBracket') = 
      #do i=1,'NumberOfModes'
         + delta(Index,ModeIndices('i'))*Sqrt(ExNumber'i'+1)* S'STATE'(GNumber,'incIndex{'i'}')
      #enddo
      ;
   id Sqrt(0)=0;   id Sqrt(1)=1; id Sqrt(NummerX?)^2 = NummerX;
 endrepeat;
.sort
 id Bra'STATE'(GNumber?,'ExBracket') * S'STATE'(GNumber?,'ExBracket') = 1;
   also S'STATE'(GNumber?,'ExBracket') = 0;
 #ifdef 'SeperateMatrixElementBySubstates'
   Bracket 'OperatorBrackets';
   .sort
   collect 'STATE'Terms;
 #endif
#endprocedure
@**************************  End of calculateFermSubstate /calculateBosSubstate  ********************************}

@*************************************  Procedure OrderMatrixElement  *******************************************{
@* USE: 	OrderMatrixElement										 *
@* DESCRIPTION:							 						 *
@* 	orders anhilation and creation operators to the appropriate substate as well as bra vectors		 *
@* 	with respect to the antikommutating nature of fermionic operators 					 *
@* 	normalorder is assumed to be implicit, so the procedure OrderMatrixElement also normaloders 		 *
@* 	the given operators while shifting them to the appropriate substate			 		 *
@*	the operators will be left with the following order:							 *
@*	     Theta4...Theta1 * Eta4...Eta1 * Betha4...Betha1 * Alpha4...Alpha1					 *
@*	where each subblock is normalordered, so for instance 							 *
@*	     Eta4= Eta4+(Mode_1)...Eta4+(Mode_k)Eta4-(Mode_k+1)...Eta4-(Mode_n)					 *
@*****************************************************************************************************************

#procedure OrderMatrixElement
  repeat;
   #ifndef 'NoBosonicOperators'
      id disorder Op1?BosOperators(Index1?)*Op2?(Index2?) = Op2(Index2)*Op1(Index1);
   #endif
   #ifndef 'NoFermionicOperators'
      id disorder Op1?FermOperators(Index1?)*Op2?FermOperators(Index2?) = -Op2(Index2)*Op1(Index1); 
   #endif
  endrepeat;
  repeat;
   #ifndef 'NoBosonicOperators'
      id disorder Op1?BosOperators(Index?)*St?(?ExNumber) = St(?ExNumber)*Op1(Index);
   #endif
   #ifndef 'NoFermionicOperators'
      id disorder Op1?FermOperators(Index?)*St?FermStates(GNumber?,?ExNumber) = sign_(GNumber)*St(GNumber,?ExNumber)*Op1(Index);
   #endif
  endrepeat;
  repeat;
      id disorder St?BraVectors(GNumber?,?ExNumber)*Op?(Index?) = Op(Index)*St(GNumber,?ExNumber);
      id disorder St1?BraVectors(GNumber1?,?ExNumber1)*St2?(GNumber2?,?ExNumber2) = St2(GNumber2,?ExNumber2)*St1(GNumber1,?ExNumber1);
  endrepeat;
#endprocedure

@***************************************  End of OrderMatrixElement  ********************************************}


@**************************************  Procedure CalculateElement  ********************************************{
@* USE: 	CalculateElement										 *
@* DESCRIPTION:	The procedure combines the matrix element of the local expressions an preprocessor variable	 * 
@*		BraVector, HamiltonianOperator and KetVector, which have to be provided. 			 *
@*		All expressions except MatrixElement are being hidden, so they don't get changed and unhidden	 *
@*		at the end of CalculateElement, so all expressions are unhidden afterwards.			 *
@*		CalculateElement uses the procedures calculateBosSubstate, calculateFermSubstate 		 *
@*		and OrderMatrixElement. 									 *
@*****************************************************************************************************************
#procedure CalculateElement
  #$Column = {'$Column' +1};
  .sort
  Drop KetVector;
  Hide; Nhide MatrixElement;
  Local MatrixElement = BraVector * 'HamiltonianOperator' * KetVector;
  #call OrderMatrixElement
 .sort
  #ifdef 'UsedFermSubstates'
    #do workoutstate = 'UsedFermSubstates'
      #call calculateFermSubstate('workoutstate')
    #enddo
 .sort 
  #endif
  #ifdef 'UsedBosSubstates'
    #do workoutstate = 'UsedBosSubstates'
      #call calculateBosSubstate('workoutstate')
    #enddo
  .sort
  #endif
  multiply left 'HamiltonianFunction';
  id many Index? = SymbolContainer(Index);
  repeat;
    id delta(Index?,NummerX?)*delta(Index?,NummerY?) = delta(Index,NummerX)*delta_(NummerX,NummerY);
  endrepeat;
    id delta_(NummerX?,NummerY?) = 0;
  repeat;
    id delta(-Index?,NummerX?)*delta(Index?,NummerX?) = delta(Index,0)*delta_(NummerX,0);
    id delta(-Index?,NummerX?)*delta(Index?,NummerY?) = delta(Index,NummerY)*delta_(NummerX,-NummerY);
  endrepeat;
  repeat;
    id func?(Index?)*delta(Index?,NummerX?) = func(NummerX)*delta(Index,NummerX);
    id func?(Index?)*delta(-Index?,NummerX?) = func(-NummerX)*delta(-Index,NummerX);
    id func?(-Index?)*delta(Index?,NummerX?) = func(-NummerX)*delta(Index,NummerX);
    id DeltaSum(?Index0,Index?,?Index1)*delta(Index?,NummerX?) = DeltaSum(?Index0,NummerX,?Index1)*delta(Index,NummerX);
    id DeltaSum(?Index0,-Index?,?Index1)*delta(Index?,NummerX?) = DeltaSum(?Index0,-NummerX,?Index1)*delta(Index,NummerX);
    id DeltaSum(?Index0,Index?,?Index1)*delta(-Index?,NummerX?) = DeltaSum(?Index0,-NummerX,?Index1)*delta(-Index,NummerX);
  endrepeat;
  id many delta(Index?,NummerX?) =1;
  id SymbolContainer(NummerX?) = NummerX;
  #call SimplificationsBeforeExport
 .sort
  #call OutputControl
  #ifdef 'WriteMatrixElementToScreen'
     #message exporting MatrixElement '$Row'x'$Column'
     Print +s;
  #endif
  .sort
  Drop MatrixElement;
  Unhide;
  #$currentMatrixElement = MatrixElement;
  #call ElementExportToMathematica('ElementName','ExportMatrixFileName','ExportStatesEquationFileName','$Row','$Column')
  .sort
#endprocedure 
@****************************************  End of CalculateElement  *********************************************}


@******************************  Procedure BeginNewHamiltonianLine & renameBra **********************************{
@* USE: 	BeginNewHamiltonianLine & renameBra								 *
@* DESCRIPTION  renameBra:		 									 *
@* 		if in the expression BraVector (provided by the C programm states.c) the BraFlag is multiplied 	 *
@*		to the expression, all names of states given in the set Ketvectors are changed into the 	 *
@*		according bra-vector of the set BraVectors (sets sould be defined in "define_states.tmp")	 *
@*		The $-variable Column is set to zero and Row is increased by one.				 *
@* DESCRIPTION: BeginNewHamiltonianLine	 									 *
@* 		This function is called every time a new line in the Hamiltoninan matrix is started.		 *
@*		 - does nothing yet - 										 *
@*****************************************************************************************************************

#procedure BeginNewHamiltonianLine
@ .sort
@ Unhide BraVector;
#endprocedure

#procedure renameBra
 #$Column = 0;
 #$Row = {'$Row' +1};
 #ifdef `VERBOSE'
     #write "            line '$Row'"
 #endif
 repeat;
    id BraFlag*St?KetVectors[NummerX](GNumber?,?Index) = BraFlag*BraVectors[NummerX](GNumber,?Index);
 endrepeat;
 id BraFlag = 1;
 .sort
@  #message in BeginNewHamiltonianLine: '$Row'x'$Column'
@ Hide BraVector;
#endprocedure
@*********************************  End of BeginNewHamiltonianLine & renameBra  *********************************}


@**********************************************  END OF MODULE   ************************************************@
