@*                                               *****************************************************************
@*                                               * FILE:        export.prc                       		 *
@*                                               * written by:  Alexander Hentschel                              *
@*                                               *              Humboldt University Berlin                       *
@*                                               *              Alexander.Hentschel@physik.hu-berlin.de          *
@*                                               *              July 2006                                        *
@*                                               ****************************************************************{


@*****************************************************************************************************************
@********************************************  Module export  ****************************************************
@*****************************************************************************************************************
@* provides the following FORM procedures:			 						 *
@*    BeginExportToMathematica('FILENAME')									 *
@*    MatrixExportToMathematica('FILENAME')									 *
@*    ElementExportToMathematica('FILENAME')									 *
@*    completeExportToMathematica('FILENAME')									 *
@*														 *
@* ------------------------------------------------------------------------------------------------------------- *
@* provides the following symbol and function Definitions:	 						 *
      #define defaultExNumberName "x"
      #define ExportMatrixFileName "Matrix_Mathematica.tmp"
      #define ExportStatesEquationFileName "StatesEquation.tmp"
      #define ExportMathematicaClosure "MathClosure.tmp"      
@****************************************************************************************************************}

@**********************************  Procedure SimplificationsBeforeExport **************************************{
@* USE: 	SimplificationsBeforeExport									 *
@* DESCRIPTION  applies several simplifications and calls procedure Simplifications				 *
@* CAUTION	procedure Simplifications has to be provided by the user					 *
@*****************************************************************************************************************

#procedure SimplificationsBeforeExport
  id many fFunction(-NummerX?) = fFunction(NummerX);
  id many gFunction(-NummerX?) = -gFunction(NummerX);
  id many InvSqrtOmega(-NummerX?) = InvSqrtOmega(NummerX);
  id many InvSqrtOmega(NummerX?)^2 = Omega(NummerX)^-1 ;
  id many Omega(-NummerX?) = Omega(NummerX);
  id many Kappa*fFunction(NummerX?)*gFunction(NummerX?)/Sqrt(Lamdatilde) = 1/2 * NummerX / Omega(NummerX);
  id many Omega(NummerX?) / Omega(NummerX?) = 1;
  .sort
   #call Simplifications
  id many fFunction(-NummerX?) = fFunction(NummerX);
  id many gFunction(-NummerX?) = -gFunction(NummerX);
  id many InvSqrtOmega(-NummerX?) = InvSqrtOmega(NummerX);
  id many InvSqrtOmega(NummerX?)^2 = Omega(NummerX)^-1 ;
  id many Omega(-NummerX?) = Omega(NummerX);
  id many Kappa*fFunction(NummerX?)*gFunction(NummerX?)/Sqrt(Lamdatilde) = 1/2 * NummerX / Omega(NummerX);
  id many Omega(NummerX?) / Omega(NummerX?) = 1;
#endprocedure
@**********************************  Procedure SimplificationsBeforeExport **************************************}
   
@*******  Procedure BeginExportToMathematica, EndMatrixExportToMathematica, completeExportToMathematica  ********{
@* USE: 	BeginExportToMathematica(ExportMatrixName,MatrixFileName,StatesEqFilenName)			 *
@*		EndMatrixExportToMathematica(MatrixFileName,StatesEqFilenName)					 *
@*		completeExportToMathematica(FILENAME)								 *
@* DESCRIPTION  			 									 *
@*   BeginExportToMathematica:											 *
@*	A matrix opening Mathematica code is written to MatrixFileName and a beginning of a RowBox environment 	 *
@*	to file StatesEqFilenName. The $-variable Row is set to zero.					 *
@*   EndMatrixExportToMathematica										 *
@*	The Matrix  and RowBox environment created by BeginExportToMathematica are closed by appending  	 *
@*	according commands to the files MatrixFileName and StatesEqFilenName.					 *
@*   completeExportToMathematica										 *
@*	writes Mathematica commands that close the input environment to the file with name FILENAME.		 *
@*	It also adds windowsize and some other control statements to the file.					 *
@*****************************************************************************************************************
            
#procedure BeginMatrixExportToMathematica(ExportMatrixName,MatrixFileName,StatesEqFilenName)
   $Row = 0;   
   #write <'MatrixFileName'> "    RowBox[{"  
   #write <'MatrixFileName'> "      RowBox[{\"'ExportMatrixName'\", \" \", \"=\","   
   #write <'MatrixFileName'> "        RowBox[{\"(\", GridBox[{"  
   #write <'StatesEqFilenName'> "    RowBox[{"  
#endprocedure
    
#procedure EndMatrixExportToMathematica(MatrixFileName,StatesEqFilenName)
   #write <'StatesEqFilenName'> "             \"\b[IndentingNewLine]\""
   #write <'StatesEqFilenName'> "    }], \"\b[IndentingNewLine]\","
   #write <'MatrixFileName'> "              }], \")\"}]}], \" \",\";\"}], \"\b[IndentingNewLine]\","   
#endprocedure

#procedure ExportToMathematicaEigenvalueCalculation(MatrixFileName,NumberOfHamiltonianParts,ExportMatrixName)
   #write <'MatrixFileName'> "    \b(Hamiltonian\b  = \b %"
   #do HP = 1, {'NumberOfHamiltonianParts'-1}
      #write <'MatrixFileName'> "'ExportMatrixName''HP'\b + \b %"
   #enddo
   #write <'MatrixFileName'> "'ExportMatrixName''NumberOfHamiltonianParts';\b),\"\b[IndentingNewLine]\", "
   #write <'MatrixFileName'> "    RowBox[{\b(Hamiltonian\b  // \b MatrixForm\b),  \"\b[IndentingNewLine]\"}], %"
   #write <'MatrixFileName'> "       \"\b[IndentingNewLine]\","
   #write <'MatrixFileName'> "    RowBox[{\b( \b(Lamdatilde\b/\b(4\b PPlus\b)\b)  Eigenvalues[Hamiltonian]\b  // \b"
   #write <'MatrixFileName'> "        Chop\b),\"\b[IndentingNewLine]\"}],%"
   #write <'MatrixFileName'> "\"\b[IndentingNewLine]\","
#endprocedure

#procedure completeExportToMathematica(FILENAME)
   #create <'FILENAME'>
   #write <'FILENAME'> "\"\b[IndentingNewLine]\""   
   #write <'FILENAME'> "}], \"Input\"]"
   #write <'FILENAME'> "},"
   #write <'FILENAME'> "FrontEndVersion->\"5.2 for X\","
   #write <'FILENAME'> "ScreenRectangle->{{0, 1280}, {0, 1024}},"
   #write <'FILENAME'> "WindowSize->{1130, 931},"
   #write <'FILENAME'> "WindowMargins->{{40, Automatic}, {Automatic, 0}}"
   #write <'FILENAME'> "]\n\n"
   #write <'FILENAME'> "(***************************************************%"
   #write <'FILENAME'> "**************************************************************"
   #write <'FILENAME'> "                                    End of Mathematica Notebook file."
   #write <'FILENAME'> "******************************************************%"
   #write <'FILENAME'> "***********************************************************)"
   #close <'FILENAME'>
#endprocedure

@*******  Procedure BeginExportToMathematica, EndMatrixExportToMathematica, completeExportToMathematica  ********}

@***********************************  Procedure ElementExportToMathematica **************************************{
@* USE: 	ElementExportToMathematica(ExportMatrixName,MatrixFileName,StatesEqFilenName,Row,Column)	 *
@* DESCRIPTION  			 									 *
@*   This procedure exports the acvtive FORM expression MatrixElement into Mathematica code.  			 *
@*   MatrixElement is understood as an matrix element wich matrix position is given by Row and Column. 		 *
@*   If MatrixElement is zero a 0 is exported as the according matrix element to the file MatrixFileName. 	 *
@*   Otherwise a symbol named 'ExportMatrixName''Row'x'Column' is exported to MatrixFileName and the definition  *
@*   of this symbol (defined as FORM expression MatrixElement) is exported to the file StatesEqFilenName.	 *
@*   The procedure BeginExportToMathematica has to be called before with the same file names, since it writes 	 *
@*   the matrix opening commands to the file MatrixFileName as well as the RowBox environment 			 *
@*   to StatesEqFilenName											 *
@*****************************************************************************************************************

#procedure ElementExportToMathematica(ExportMatrixName,MatrixFileName,StatesEqFilenName,Row,Column)
   #append <'MatrixFileName'>
   #if ('Column' == 1 )
      #write <'MatrixFileName'> "             {%"
   #endif
   #if ("'$currentMatrixElement'" != "0")
             #write <'MatrixFileName'> "\b('ExportMatrixName'\b_\b('Row', 'Column'\b)\b)%"
      #else
         #write <'MatrixFileName'> "\"0\"%"
   #endif
   #if ('Column' != 'NumberOfStates' )
      #write <'MatrixFileName'> ", %"
   #endif
   #if ('Column' == 'NumberOfStates')
      #if ('Row' == 'NumberOfStates')
         #write <'MatrixFileName'> "}"
      #else
         #write <'MatrixFileName'> "},"
      #endif
   #endif
   #if ("'$currentMatrixElement'" != "0")
     #write <'StatesEqFilenName'> "             \b('ExportMatrixName'\b_\b('Row', 'Column'\b)\b  =  %E;\b), \"\b[IndentingNewLine]\",",MatrixElement
   #endif
#endprocedure
@***********************************  Procedure ElementExportToMathematica **************************************}

@**********************************************  END OF MODULE   ************************************************@
