#-
@*                                               *****************************************************************
@*                                               * FILE:        computeHamiltonian.frm                           *
@*                                               * written by:  Alexander Hentschel                              *
@*                                               *              Humboldt University Berlin                       *
@*                                               *              Alexander.Hentschel@physik.hu-berlin.de          *
@*                                               *              Septermber 2006                                  *
@*                                               *****************************************************************

@ COMMENT:
@ please set "commentchar @" in file "form.set" indicating that comments begin with "@" 
@ but in "form.set" the character "*" remains as the comment sign

@ DESCRIPTION:
format 160;

#include define_Hamiltonian.load
#include Hamiltonian.h

CFunction DF, DF2;
NFunction NDF, NDF1, NDF2, NDF3, NDF4;
Symbols Ia,Ib, sumIndex;

#include computeHamiltonian.prc
#include Hamiltonian.def

#write  " CALCULATING EFFECTIVE HAMILTONIAN OPERATOR (done by FORM-program computeHamiltonian.frm):"
@ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ generate Hamiltonian - Part 1 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@ ~ > drop all terms of Z,PZ,Y,PY,Eta & Theta of which the whole operator class is not used 			 ~
 #call SetNotUsedOperatorClassesToZero(Alpha,Y,Yprime,PY,PYprime)
 #call SetNotUsedOperatorClassesToZero(Beta,Z,Zprime,PZ,PZprime)
 #call SetNotUsedOperatorClassesToZero(Theta,Theta,Thetaprime,Thetadeg,Thetadegprime)
 #call SetNotUsedOperatorClassesToZero(Eta,Eta,Etaprime,Etadeg,Etadegprime)
 .sort
 
@ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ generate Hamiltonian - Part 2 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@ ~ > sum over indices of Z, Y, PZ & PY 									 ~
 splitarg Y, Yprime, Z, Zprime;
 id Y(Ia?,-Ib?) = Ydeg(5-Ia,Ib);
 id Yprime(Ia?,-Ib?) = Ydegprime(5-Ia,Ib);
 id Z(Ia?,-Ib?) = Zdeg(5-Ia,Ib);
 id Zprime(Ia?,-Ib?) = Zdegprime(5-Ia,Ib);
 id many DF?{Ydeg,Ydegprime,Zdeg,Zdegprime}(0,Ia?) = DF(Ia);
 if (match( Ydeg(Ia?,Ib?) ));      Print ""; Print "ERROR:  Field Y(i) with index i>4 detected!";   exit ""; endif;
 if (match( Ydegprime(Ia?,Ib?) )); Print ""; Print "ERROR:  Field Y\'(i) with index i>4 detected!"; exit ""; endif;
 if (match( Zdeg(Ia?,Ib?) ));      Print ""; Print "ERROR:  Field Z(i) with index i>4 detected!";   exit ""; endif;
 if (match( Zdegprime(Ia?,Ib?) )); Print ""; Print "ERROR:  Field Z\'(i) with index i>4 detected!"; exit ""; endif;
 id many DF?(Ia?symbol_)*DF2?(Ia?symbol_) = sum_(sumIndex,1,4, DF(sumIndex)*DF2(sumIndex) );
 id Gamma(Ia?)*Gamma(Ia?) = 0;
 .sort

@ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ generate Hamiltonian - Part 3 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@ ~ > substitute matrix representation (in terms of EtaTildePlus, EtaTildeMinus, ThetaTildePlus, ThetaTildeMinus)~ 
@ ~   of Eta, Etadeg, Theta, Thetadeg										 ~
@ ~ > drop non used operator functions of type EtaTildePlus, EtaTildeMinus, ThetaTildePlus & ThetaTildeMinus	 ~
 #write  "   MESSAGE: substitute matrix representation of fields and trace matrices" 
 id Eta =      sum_(sumIndex,1,4,      Gamma(5-sumIndex)*EtaTildeMinus(sumIndex,fFunction) 
                                     + KomplexI*Gamma(sumIndex)*EtaTildePlus(sumIndex,gFunction)          );
 id Etadeg =   sum_(sumIndex,1,4,      Gamma(sumIndex)*EtaTildePlus(sumIndex,fFunction) 
                                     + KomplexI*Gamma(5-sumIndex)*EtaTildeMinus(sumIndex,gFunction)       );
 id Etaprime = sum_(sumIndex,1,4,      Gamma(5-sumIndex)*EtaTildeMinusPrime(sumIndex,fFunction) 
                                     + KomplexI*Gamma(sumIndex)*EtaTildePlusPrime(sumIndex,gFunction)     );
 id Etadegprime = sum_(sumIndex,1,4,   Gamma(sumIndex)*EtaTildePlusPrime(sumIndex,fFunction) 
 		  		     + KomplexI*Gamma(5-sumIndex)*EtaTildeMinusPrime(sumIndex,gFunction)  );
 id Theta =       sum_(sumIndex,1,4,   Gamma(5-sumIndex)*ThetaTildeMinus(sumIndex,fFunction) 
 				     + KomplexI*Gamma(sumIndex)*ThetaTildePlus(sumIndex,gFunction)        );
 id Thetadeg =    sum_(sumIndex,1,4,   Gamma(sumIndex)*ThetaTildePlus(sumIndex,fFunction) 
 				     + KomplexI*Gamma(5-sumIndex)*ThetaTildeMinus(sumIndex,gFunction)     );
 id Thetaprime =  sum_(sumIndex,1,4,   Gamma(5-sumIndex)*ThetaTildeMinusPrime(sumIndex,fFunction) 
                 		     + KomplexI*Gamma(sumIndex)*ThetaTildePlusPrime(sumIndex,gFunction)   );
 id Thetadegprime =sum_(sumIndex,1,4,  Gamma(sumIndex)*ThetaTildePlusPrime(sumIndex,fFunction) 
 				     + KomplexI*Gamma(5-sumIndex)*ThetaTildeMinusPrime(sumIndex,gFunction));
@ #call DropNotUsedFermionicOperators(Theta,ThetaTildePlus,ThetaTildeMinus,ThetaTildePlusPrime,ThetaTildeMinusPrime)
@ #call DropNotUsedFermionicOperators(Eta,EtaTildePlus,EtaTildeMinus,EtaTildePlusPrime,EtaTildeMinusPrime)
 #call DropNotUsedOperators(Theta,ThetaTildePlus,ThetaTildeMinus,ThetaTildePlusPrime,ThetaTildeMinusPrime)
 #call DropNotUsedOperators(Eta,EtaTildePlus,EtaTildeMinus,EtaTildePlusPrime,EtaTildeMinusPrime)

@ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ generate Hamiltonian - Part 4 & Part 5 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@ ~ > apply simplifications on base of the Gamma matrices							 ~
@ ~ > substitute MatrixPplus, MatrixPminus, MatrixSigma & Gamma by their Dirac-gamma-matrix representation 	 ~
@ ~ > trace over Dirac-gamma-matrices (in FORM: g_)							 	 ~
 repeat;
    id disorder NDF?FermionicTildeFields(Ib?,DF?)*Gamma(Ia?) = Gamma(Ia)*NDF(Ib,DF);
 endrepeat;
 repeat;
   id Gamma(Ia?)*Gamma(Ia?) = 0;
   id disorder Gamma(1)*Gamma(2) = - Gamma(2)*Gamma(1); id disorder Gamma(2)*Gamma(1) = - Gamma(1)*Gamma(2);
   id disorder Gamma(1)*Gamma(3) = - Gamma(3)*Gamma(1); id disorder Gamma(3)*Gamma(1) = - Gamma(1)*Gamma(3);
   id disorder Gamma(4)*Gamma(2) = - Gamma(2)*Gamma(4); id disorder Gamma(2)*Gamma(4) = - Gamma(4)*Gamma(2); 
   id disorder Gamma(4)*Gamma(3) = - Gamma(3)*Gamma(4); id disorder Gamma(3)*Gamma(4) = - Gamma(4)*Gamma(3); 
 endrepeat;
 id KomplexI^2 = -1; 
 .sort
 id MatrixPplus  = 1/2*(gi_(1) + MatrixSigma);
 id MatrixPminus = 1/2*(gi_(1) - MatrixSigma);
 id MatrixSigma = -g_(1, 1,2,3,4);
 id many Gamma(1) = 1/2*(g_(1, 2) -KomplexI* g_(1, 1));
 id many Gamma(2) = 1/2*(g_(1, 4) -KomplexI* g_(1, 3));
 id many Gamma(3) = 1/2*(g_(1, 4) +KomplexI* g_(1, 3));
 id many Gamma(4) = 1/2*(g_(1, 2) +KomplexI* g_(1, 1));
 trace4 1;
 id KomplexI^2 = -1; 
 .sort 
 if (match( Gamma(Ia?) ));  Print ""; Print "ERROR:  Internal bug - after tracing still Gamma-matrices found!";   
 				      Print "        Please report bug to hentsche[at]physik.hu-berlin.de"; exit ""; endif;
  
@ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ generate Hamiltonian - Part 6 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@ ~ > substitute decomposition of Z, Y, PZ, PY into creation and anhilation operators 				 ~
@ ~ > order fermionic operators using anticommutator relation (Hamiltonian implicite normalordered)		 ~
@ ~   bosonic operators are ordered by Form by default								 ~ 
@ ~ > drop non used operator functions of type Y, Yprime, Z & ZPrime 						 ~
@ ~ > substitute mode decomposition of Alpha, Beta, Theta and Eta						 ~
 #write  "   MESSAGE: substitute mode decomposition of fields and drop not contributing terms" 
 id many Ydeg(Ia?) = Y(5-Ia);
 id many Ydegprime(Ia?) = Yprime(5-Ia);
 id many Zdeg(Ia?) = Z(5-Ia);
 id many Zdegprime(Ia?) = Zprime(5-Ia);
 repeat;
   id disorder NDF?FermionicTildeFields(Ia?,DF?)*NDF2?FermionicTildeFields(Ib?,DF2?) = -NDF2(Ib,DF2)*NDF(Ia,DF);
 endrepeat;
 id many Z(Ia?)       = -KomplexI * BetaTildePlus(Ia, -1)      + KomplexI * BetaTildeMinus(5-Ia,-1);
 id many Zprime(Ia?)  = -KomplexI * BetaTildePlusPrime(Ia, -1) + KomplexI * BetaTildeMinusPrime(5-Ia,-1);
 id many PZ(Ia?)      = 1/2 * BetaTildePlus(Ia, +1)      + 1/2 * BetaTildeMinus(5-Ia,+1);
 id many PZprime(Ia?) = 1/2 * BetaTildePlusPrime(Ia, +1) + 1/2 * BetaTildeMinusPrime(5-Ia,+1);
 id many Y(Ia?)       = -KomplexI * AlphaTildePlus(Ia, -1)      + KomplexI * AlphaTildeMinus(5-Ia, -1);
 id many Yprime(Ia?)  = -KomplexI * AlphaTildePlusPrime(Ia, -1) + KomplexI * AlphaTildeMinusPrime(5-Ia, -1);
 id many PY(Ia?)      = 1/2 * AlphaTildePlus(Ia, +1)      + 1/2 * AlphaTildeMinus(5-Ia, +1);
 id many PYprime(Ia?) = 1/2 * AlphaTildePlusPrime(Ia, +1) + 1/2 * AlphaTildeMinusPrime(5-Ia, +1);
 id KomplexI^4 = 1;
 id KomplexI^2 = -1;
 #call DropNotUsedOperators(Alpha, AlphaTildeMinusPrime, AlphaTildeMinus, AlphaTildePlusPrime, AlphaTildePlus)
 #call DropNotUsedOperators(Beta, BetaTildeMinusPrime, BetaTildeMinus, BetaTildePlusPrime, BetaTildePlus)
 #call SubstituteModeDecomposition
 
@ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ process Hamiltonian - Part 1 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@ ~ > rewrite operator names in terms of FORM matrix calculation						 ~
@ ~   (i.e. flavour numbers are included in operator names)							 ~
@ ~ > hide local functions [Hbb], [Hbf-Part1], ..., [Hbf-Part6], [Hff-Part1], [Hff-Part2]			 ~
@ ~ > for every suitable operator configuration: create function and operator part		 		 ~
 #write  "   MESSAGE: Hamiltonian segmentation:" 
 repeat;
   id Exp(?Ia)*Exp(?Ib) = Exp(?Ia,?Ib);
 endrepeat;
 #call RewriteOperators(Theta)
 #call RewriteOperators(Eta)
 #call RewriteOperators(Beta) 
 #call RewriteOperators(Alpha)
.sort 
 repeat;
   #ifndef `NoFermionicOperators'
      id disorder NDF?FermOperators(Ia?)*NDF2?FermOperators(Ib?) = -NDF2(Ib)*NDF(Ia);
   #endif
   #ifndef `NoBosonicOperators'
     id disorder NDF?BosOperators(Ia?)*NDF2?BosOperators(Ib?)   =  NDF2(Ib)*NDF(Ia);
     #ifndef `NoFermionicOperators'
        id disorder NDF?BosOperators(Ia?)*NDF2?FermOperators(Ib?)  =  NDF2(Ib)*NDF(Ia);
     #endif
   #endif
 endrepeat;
 #call DropNonSectorParts
 #call HamiltonianSegmentation
 #call checkHamiltonianSegmentation

@ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ process Hamiltonian - Part 2 ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
@ ~ > export Hamiltonian into FORM file imported to 
 #call exportHamiltonianToForm(`HamiltonianExportFile')
 #write  "   MESSAGE: effective Hamiltonian written to file \"`HamiltonianExportFile'\"" 
 .end
 
@ End of FORM-FILE
