/*                                               *****************************************************************
//                                               * FILE:        recycleInput.c                                   *
//                                               * written by:  Alexander Hentschel                              *
//                                               *              Humboldt University Berlin                       *
//                                               *              Alexander.Hentschel@physik.hu-berlin.de          *
//                                               *              July 2006                                        *
//                                               ****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <limits.h>
#include <math.h>

#include "define_states.def"
#include "recycleInput.h"

static int ModeNamesGenerated = 0; 	/* 0 = no, 1=yes */
static int total_excitations = -1;
static int NumberOfFirstBosOperator = -10;
static int NoBosOperatorWarning = 0;
static int OperatorOrderChanged = 0;	/* 0 = no, 1=yes */
static int OrOp[MAXOP];

/* Funktion today()
 * return value: String der aktuelles Datum und Uhrzeit enthaelt */
char* today(void) {
  struct tm *tmnow;
  time_t tnow;
  time(&tnow);
  tmnow = localtime(&tnow);
  return(asctime(tmnow));
} /* end of function today */

/* Funktion getIndexOfFirstBosOperator()
 * testet, ob jeder unter BosOperators angegebene Operator auch in DefOperatorOrder gelistet ist
 * ermittelt die Nummer des ersten bosonischen Operators im Array Operators und gibt diese zuruck
 * falls kein bosonischer Operator enthalten ist, liefert getIndexOfFirstBosOperator() den Wert -1 */
int getIndexOfFirstBosOperator(char* Operators[], int NumberOfOperators){
  if (NumberOfFirstBosOperator > -10) return(NumberOfFirstBosOperator);
  #ifndef DefOperatorOrder
     printf("\nERROR: No operation order defined in \"recycleInput.h\"\n\n"); exit(1);
  #endif
  #ifndef BosOperators
     if (NoBosOperatorWarning == 0) {
       printf("   WARNING: No bosonic operators defined in \"recycleInput.h\",\n"); 
       printf("            assuming there are none.\n");
       printf("            This warning is only printed once. \n");
     }
     NoBosOperatorWarning = 1;
     return(NumberOfOperators);
  #endif
  NoBosOperatorWarning = 1;
  #ifdef DefOperatorOrder
    #ifdef BosOperators 
      { /* bracket 1 */
       int i,j,NumberOfBosOp,NumberOfOrderedOp;
       char *OpOrder[] = DefOperatorOrder;
       {/* bracket 2 */
          char *BosOp[] = BosOperators;	
	  NoBosOperatorWarning = 1;
	  NumberOfOrderedOp = sizeof(OpOrder)/sizeof(*OpOrder);  
	  NumberOfBosOp = sizeof(BosOp)/sizeof(*BosOp);
	  for(i=0; i< NumberOfBosOp; i++) { /* test if all bosonic operators are listed in "DefOperatorOrder" */
  	    for(j=0; j< NumberOfOrderedOp; j++) 
	      if  (strcmp(BosOp[i],OpOrder[j])==0) break; 
	    if(j== NumberOfOrderedOp) {printf("\nERROR: bosonic operator \"%s\" of \"BosOperators\" not listed in \"DefOperatorOrder\"!\n\n",BosOp[i]); exit(1);}
	  } /* end of for(i=0; i< NumberOfBosOp; i++) */
	  for(i=0; i< NumberOfOperators; i++) /* search for first bosonic operator */
 	    for(j=0; j< NumberOfBosOp; j++) 
	      if (strcmp(Operators[i],BosOp[j])==0) { 
	        NumberOfFirstBosOperator = i;
	        return(i);
	      }
	  NumberOfFirstBosOperator = NumberOfOperators;   
        } /* end of bracket 1 */ } /* end of bracket 1 */
    #endif
  #endif
  return(NumberOfOperators);
} /* end of function getIndexOfFirstBosOperator() */

/* Funktion OrderOperators()
 * Sortiert Operatoren in Reihenfolge, wie sie in OperatorOrder angegeben sind
 * return value: Anzahl Operatoren nach entfernen von Duplikaten */
int OrderOperators(char* UnorderedOp[], int NumberOfOperators){
  int NumberOfOrderedOp;
  char* OpOrdered[MAXOP];
  int index=0; 
  int i,j;
  char* OpOrder[] = DefOperatorOrder;
  #ifndef DefOperatorOrder
     printf("\nERROR: No operation order defined in \"recycleInput.h\"\n\n"); exit(1);
  #endif
  NumberOfFirstBosOperator = -10;  	/* set value of NumberOfFirstBosOperator to 'unset' */
  OperatorOrderChanged = 0;		/* set value of OperatorOrderChanged to 'unchanged' */
  printf("   MESSAGE: ");
  if (UnorderedOp == NULL) return(0);
  NumberOfOrderedOp = sizeof(OpOrder)/sizeof(*OpOrder);
  for(i=0; i< MAXOP; OrOp[i++]=-1);
  for(i=0; i< NumberOfOperators; i++) {
      for(j=0; j< NumberOfOrderedOp; j++) {
        if  (strcmp(UnorderedOp[i],OpOrder[j])==0) {OrOp[j]=i; break;} /* match found */
      }  
      if (j == NumberOfOrderedOp) {printf("\nERROR: unknown creation operator: \"%s\"\n\n",UnorderedOp[i]); exit(1); } /* not match */
  } /* end of for(i=0; i< NumberOfOperators; i++) */
  for(i=0; i< MAXOP; i++){ if (OrOp[i] > -1) OpOrdered[index++] = UnorderedOp[OrOp[i]]; }
  if (index < NumberOfOperators) {
      printf("   WARNING: Some creation operators are listed more than once!\n");
      printf("            In calculation only used once!\n");
  }
  NumberOfOperators = index;
  for(i=0; i< NumberOfOperators; UnorderedOp[i]= OpOrdered[i++]);
  printf("number of operators = %d\t", NumberOfOperators);
  if (NumberOfOperators>MAXOP){ printf("\nERROR: more than %d possible creation Operators!\n\n",MAXOP); exit(1);}
  getIndexOfFirstBosOperator(UnorderedOp,NumberOfOperators);
  j = OrOp[0];				/* check if order has been changed: j last found value */
  if (NumberOfOperators>1)  
    for(i=1; i<MAXOP; i++) 		/* checke jdes elements */
       if(OrOp[i] > -1) {		/* ob es einem zugelassenen Operator entspricht, falls ja */
          if (j > OrOp[i]) 		/* muss dessen Index groesser sein, als der letzte, sonst wurde Reihenfolge geaendert */
	     {OperatorOrderChanged = 1; 
	      #ifdef VERBOSE
	         printf("\n            Operator order changed to normal order\n            ");
	      #endif 
	      break; }
	  j = OrOp[i];
       } /* end of if(OrOp[i] > -1) */
  return(NumberOfOperators);
} /* end of function OrderOperators */

char** checkExitations_and_Modes(char* Modes[],int NumberOfModeElements, int ExcNumbers[], int NumberOfExcElements, int NumberOfOperators) {
  int i=0;
  int a,b;
  int numberOfchar;
  if (Modes != NULL) { /* if mode indices are defined, check if number is consistent with number of modes */
    if (NumberOfModeElements != NumberOfExcElements) { 
      printf("\nERROR: number of elements in \"ModeIndices\" (%d) not matching ",NumberOfModeElements);
      printf("\n       number of elements in \"ExcitationNumbers\"   (%d)!\n\n",NumberOfExcElements);
      exit(1);
    }
    for(a=0; a<(NumberOfModeElements-1);a++ ) /* check, that no mode index is appearing more than once */ 
      for(b=a+1;b<NumberOfModeElements; b++) 
        if (strcmp(Modes[a],Modes[b])==0){
           printf("\nERROR: Mode index \"%s\" given twice in \"ModeIndices\"\n\n",Modes[a]); exit(1); }    
  } /*End of if (Modes != NULL) */
  printf("number of modes = %d\n",NumberOfExcElements);
  if (Modes == NULL) { /* if mode indices aren't defined: generate needed number*/
     numberOfchar = (int) log10(NumberOfExcElements);
     numberOfchar +=3;
     Modes = (char **) malloc(NumberOfExcElements*sizeof(char*));
     for(i=0; i< NumberOfExcElements; i++){
        Modes[i] = (char *) malloc(numberOfchar*sizeof(char));
	sprintf(Modes[i],"%s%d",DefaultModeName,i+1);
     }
     #ifdef MathematicaFile
       printf("   MESSAGE: indices %s - %s generated \n",Modes[0],Modes[NumberOfExcElements-1]);
     #endif
     ModeNamesGenerated = 1; 
  }
  total_excitations = 0;
  for(i=0; i < NumberOfExcElements; i++) {
    if (ExcNumbers[i] < 1) {
      printf("\nERROR: only numbers greater than zero in  \"ExcitationNumbers\" allowed");
      printf("\n       element %d has value of %d  \n\n",i+1,ExcNumbers[i]);
      exit(1);
    }
    total_excitations += ExcNumbers[i];
  }
  #if ((defined UseOperators) && (!defined DefineCharges))
    if (NumberOfFirstBosOperator == -10) {
      printf("\nERROR: please call function \"OrderOperators()\" of this package before");
      printf("\n       using function \"checkExitations_and_Modes()\".\n\n");
      exit(1);
    }
    if (NumberOfOperators == NumberOfFirstBosOperator)  /* only fermionic operators to create state of */
     for(i=0; i<NumberOfExcElements; i++)  /* jeder Operator darf nur eine Anregung jeder (grassmannwertigen) Mode tragen :*/
        if (ExcNumbers[i] > NumberOfOperators){ /* erlaubt: maximal so viele Anregungen einer Mode wie Operatoren verfuegbar */
	   printf("\nERROR: Only fermionic creation operators available.\n");
	   printf("       Element %d of \"ExcitationNumbers\" defines a mode which is \n", i+1);
	   printf("       excited %d times. But it is impossible to create non vanishing\n",ExcNumbers[i]);
	   printf("       states by distributing %d excitations over %d operator flavours.\n",ExcNumbers[i],NumberOfOperators);
	   printf("       Error in function gen_all_States. \n\n");
	   exit(1);
         }
  #endif 
 return(Modes);
} /* end of function checkInput */

void checkSector(int* Sector, int NumbeOfSectorElements, int NumberOfOperators, int NumberOfModes) {
 int i,temp=0;
 int index = 0;
 int OrderedSector[MAXOP];
  if (total_excitations < 0) { 
      printf("\nERROR: please call function \"checkExitations_and_Modes()\" befor using function \"checkSector()\"!\n\n");
      exit(1);
  }
  if (Sector == NULL) return;
  if (NumbeOfSectorElements != NumberOfOperators) { 
      printf("\nERROR: number of elements in \"HamiltonianSector\" (%d) does not match ",NumbeOfSectorElements);
      printf("\n       number of Operators (%d)!\n\n",NumberOfOperators);
      exit(1);
  }
  for(i=0; i < NumberOfOperators; i++) {
    if (Sector[i] < 0) {
      printf("\nERROR: only non negative numbers allowed in \"HamiltonianSector\"");
      printf("\n       element %d has value of %d  \n\n",i+1,Sector[i]);
      exit(1);
    }
   temp += Sector[i];
  }
  if (temp != total_excitations) {
      printf("\nERROR: sum of all excitations in \"HamiltonianSector\" not equal");
      printf("\n       to sum of excitations in \"ExcitationNumbers\"\n\n");
      exit(1);
  }
  if (NumberOfFirstBosOperator == -10) {
      printf("\nERROR: please call function \"OrderOperators()\" of this package before\n");
      printf("\n       using function \"checkInput()\".\n\n");
      exit(1);
  }
  /* reorder Sector acording to the order done by OrderOperators() */
  for(i=0; i< MAXOP; i++)
    if (OrOp[i] > -1) { 
      OrderedSector[index] = Sector[OrOp[i]]; 
      /* fermion. Op. duerfen nur maximal NumberOfModes Anregungen tragen */
      if ((index < NumberOfFirstBosOperator) && (OrderedSector[index] > NumberOfModes)) {
	printf("\nERROR: Mode %d in \"HamiltonianSector\" refers to a fermionic operator flavour.\n",OrOp[i]+1);
	printf("       This operator flavour is ment to carry %d modes but can carry at most %d\n",Sector[i],NumberOfModes);
	printf("       without defining only vanishing states!\n");
	printf("       Error in function gen_States_of_One_Sector. \n\n");
	exit(1);
      }
      ++index;
    }
  for(i=0; i < NumberOfOperators; Sector[i] = OrderedSector[i++]);
} /* end of function checkSector */

void reorderSecor(int* Sector, int NumberOfOperators){
  int i,index = 0;
  int OrderedSector[MAXOP];
  if ((OperatorOrderChanged == 0) || (Sector == NULL)) return;
  for(i=0; i< MAXOP; i++) if (OrOp[i] > -1) OrderedSector[index++] = Sector[OrOp[i]]; 
  for(i=0; i < NumberOfOperators; Sector[i] = OrderedSector[i++]);
} /* end of reorderSecor() */

char** freeUsedMem(char* Modes[], int NumberOfModes){
  int i;
  if  (ModeNamesGenerated == 1) { /* if mode indices haven't been defined and generated by checkInput*/
     for(i=0; i< NumberOfModes; i++) free(Modes[i]);
     free(Modes);
  }
  return(NULL);
} /* end of function freeUsedMem() */
